from fastapi import Depends, FastAPI, HTTPException
from sqlalchemy.orm import Session

from . import queries, models, schemas
from .database import SessionLocal, engine
from .logging import logger

models.Base.metadata.create_all(bind=engine)

app = FastAPI()

# Dependency
def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()

@app.on_event("startup")
async def read_config_and_populate_db():
    # XXX Get filename from environment.
    config = schemas.config.from_yaml("config.yaml")

    db = SessionLocal()
    try:
        queries.update_from_config(db, config)
    finally:
        db.close()


@app.get("/", response_model=list[schemas.Task])
async def read_tasks(skip: int = 0, limit: int = 100, db: Session = Depends(get_db)):
    users = queries.list_tasks(db, limit)
    return users
