from sqlalchemy.orm import Session
from sqlalchemy import exists

from . import schemas
from .models import Task
from .logging import logger
from urllib.parse import urljoin


def list_tasks(db: Session, limit: int = 100):
    return db.query(models.Task).limit(limit).all()


def update_from_config(db: Session, config: schemas.Config):
    for website in config.websites:
        domain = str(website.domain)
        for p in website.paths:
            url = urljoin(domain, str(p.path))
            for check in p.checks:
                for check_key, expected in check.items():
                    # Check the db for already existing tasks.

                    existing_task = db.query(exists().where(
                        Task.url == url
                        and Task.check == check_key
                        and Task.expected == expected
                    )).scalar()
                    
                    if not existing_task:
                        task = Task(
                            domain = domain,
                            url = url,
                            check = check_key,
                            expected = expected
                        )
                        logger.debug(f"Adding a new task in the db: {task=}")
                        db.add(task)
                    else:
                        logger.debug(f"Skipping db task creation for {url=}, {check_key=}, {expected=}.")
    db.commit()
